(* runtime configuration *)
(* solyga@gmx.de, 2022-10-01 *)
IMPLEMENTATION MODULE Config;

IMPORT Str, FIO, FIOStd, Lib2, Lib3, Args, SO, Utils, ConSigs;

CONST
  prg = 'lrgcards';
  ver = 7;
  ext = '2022-10-01';

TYPE
  TVldAct = TAction[ TAction(1) .. MAX(TAction) ];
  TActStr = ARRAY [0..2] OF CHAR;
  TActStrs = ARRAY TAction OF TActStr;

CONST
  actStrs = TActStrs{ 'nul', 'tst', 'add', 'sub', 'mul', 'div', 'mod' };


PROCEDURE WrInfo( VAR b: SO.TBuffer );
BEGIN
  SO.WrStr( b, prg );
  SO.WrStr( b, ' v' );
  SO.WrCard( b, ver, 0 );
  SO.WrStr( b, ' (' );
  SO.WrStr( b, ext );
  SO.WrStr( b, '): Testbed for large cardinal arithmetics.' );
END WrInfo;

PROCEDURE WrActs( VAR b: SO.TBuffer );
VAR a: TVldAct;
BEGIN
  FOR a := MIN( TVldAct ) TO MAX( TVldAct ) DO
    SO.WrChar( b, "'" );
    SO.WrStr( b, actStrs[a] );
    SO.WrChar( b, "'" );
    IF a < MAX( TVldAct ) THEN SO.WrStr( b, " | " ) END;
  END;
END WrActs;


PROCEDURE ParseArgs();

  PROCEDURE Help();
  CONST
    f ::= FIOStd.out;
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.Clear( b ); SO.WrStr( b, "(* " ); WrInfo( b ); SO.WrStr( b, " *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "syntax = '" ); SO.WrStr( b, prg ); SO.WrStr( b, "' { option } [ action [ num num ] ] ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "option = '-h'      (* show this info and exit *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "       | '-b' num  (* set base to num *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "       | '-d'      (* switch on debug mode *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "       | '-v'      (* switch on verbose mode *) ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "action = " ); WrActs( b ); SO.WrStr( b, " ." );SO.WrLn( b );
    FIO.WrStr( f, s );
    Lib3.SetReturnCode( 0 );
    HALT;
  END Help;

  PROCEDURE Error( t-: ARRAY OF CHAR );
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.WrStr( b, 'Error: ' );
    SO.WrStr( b, t );
    IF Args.len > 0 THEN
      SO.WrStr( b, ": '" );
      SO.WrStr( b, Args.lit );
      SO.WrStr( b, "'" );
    END;
    SO.WrLn( b );
    SO.WrStr( b, "Try '" );
    SO.WrStr( b, prg );
    SO.WrStr( b, " -h' for help." );
    Lib3.FatalError( s );
  END Error;

  PROCEDURE GetAddress( VAR x: Types.TLngCard );
  BEGIN
    IF ( Args.sym # Args.symArg ) OR ~ Utils.MkIP( x, Args.lit ) THEN Error( 'Invalid IP address' ) END;
  END GetAddress;

  PROCEDURE GetPort( VAR p: Types.TCard );
  VAR
    x: Types.TLngCard;
    ok: BOOLEAN;
  BEGIN
    IF Args.sym # Args.symArg THEN Error( 'Invalid port number' ) END;
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok THEN Error( 'Invalid port number' ) END;
    IF x > MAX( Types.TCard ) THEN Error( 'Port number out of range' ) END;
    p := Types.TCard( x );
  END GetPort;

  PROCEDURE GetCard( VAR x: LONGCARD );
  VAR
    ok: BOOLEAN;
  BEGIN
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok THEN Error( 'Invalid number' ) END;
  END GetCard;

  PROCEDURE Common( s-, t-: ARRAY OF CHAR ): CARDINAL;
  VAR
    l, i: CARDINAL;
  BEGIN
    l := Str.Length( s );
    i := Str.Length( t );
    IF l > i THEN l := i END;
    i := 0;
    WHILE i < l DO
      IF CAP( s[i] ) # CAP( t[i] ) THEN RETURN i END;
      INC( i );
    END;
    RETURN l;
  END Common;

  PROCEDURE GetAction( VAR x: TAction );
  VAR
    a: TVldAct;
    n, m: CARDINAL;
  BEGIN
    x := aNull;
    m := 0;
    FOR a := MIN( TVldAct ) TO MAX( TVldAct ) DO
      n := Common( actStrs[a], Args.lit );
      IF m < n THEN m := n; x := a END;
    END;
    IF x = aNull THEN Error( 'Invalid action' ) END;
  END GetAction;

  PROCEDURE GetNum( VAR x: ARRAY OF CHAR );
  BEGIN
    Str.Copy( x, Args.lit );
  END GetNum;

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF    Str.Compare( Args.lit, '?' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'h' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'b' ) = 0 THEN
        Args.Read();
        GetCard( data.bas );
      ELSIF Str.Compare( Args.lit, 'd' ) = 0 THEN
        data.dbg := TRUE;
      ELSIF Str.Compare( Args.lit, 'v' ) = 0 THEN
        data.vrb := TRUE;
      ELSE
        Error( 'Invalid option' );
      END;
      Args.Read();
    END;
  END Options;

  PROCEDURE Action();
  BEGIN
    IF Args.sym # Args.symArg THEN RETURN END;
    GetAction( data.act );
    Args.Read();
  END Action;

  PROCEDURE Arguments();
  BEGIN
    IF data.act < aAdd THEN RETURN END;
    IF Args.sym # Args.symArg THEN Error( '1st number expected' ) END;
    Str.Copy( data.arg, Args.lit );
    Args.Read();
    IF Args.sym # Args.symArg THEN Error( '2nd number expected' ) END;
    Str.Copy( data.brg, Args.lit );
    Args.Read();
  END Arguments;

BEGIN
  Args.Reset();
  Args.Read();
  Options();
  Action();
  Options();
  Arguments();
  Options();
  IF Args.sym # Args.symEOA THEN Error( 'Unexpected argument' ) END;
END ParseArgs;

PROCEDURE InitDir();
VAR
  drv: SHORTCARD;
BEGIN
  drv := FIO.GetDrive();
  FIO.GetDir( drv, data.dir );
  Str.Prepend( data.dir, ':' );
  Str.Prepend( data.dir, CHR( ORD( 'A' ) + CARDINAL(drv-1) ) );
END InitDir;

PROCEDURE Show();
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;

  PROCEDURE Out();
  CONST f ::= FIOStd.out;
  BEGIN
    SO.WrLn( b );
    FIO.WrStr( f, s );
  END Out;

  PROCEDURE ShowHdr();
  BEGIN
    SO.Clear( b ); WrInfo( b ); Out();
    SO.Clear( b ); SO.WrStr( b, 'config' ); Out();
  END ShowHdr;

  PROCEDURE Begin( n-: ARRAY OF CHAR );
  CONST w = 3;
  BEGIN
    SO.Clear( b ); SO.WrStr( b, '  ' ); SO.WrStrAdj( b, n, -w ); SO.WrChar( b, ' ' );
  END Begin;

  PROCEDURE ShowStr( n-: ARRAY OF CHAR; v-: ARRAY OF CHAR );
  BEGIN
    Begin( n ); SO.WrChar( b, "'" ); SO.WrStr( b, v ); SO.WrChar( b, "'" ); Out();
  END ShowStr;

  PROCEDURE ShowCard( n-: ARRAY OF CHAR; v: CARDINAL );
  CONST w = 1;
  BEGIN
    Begin( n ); SO.WrLngCard( b, v, w ); Out();
  END ShowCard;

  PROCEDURE ShowIP( n-: ARRAY OF CHAR; x: Types.TLngCard );
  BEGIN
    Begin( n ); Utils.WrIP( b, x ); Out();
  END ShowIP;

BEGIN
  IF ~ data.vrb THEN RETURN END;
  SO.Init( b, s );
  ShowHdr();
  ShowStr(  'prg', data.prg );
  ShowCard( 'ver', data.ver );
  ShowStr(  'ext', data.ext );
  ShowStr(  'dir', data.dir );
  ShowCard( 'dbg', ORD( data.dbg ) );
  ShowCard( 'vrb', ORD( data.vrb ) );
  ShowCard( 'bas', data.bas );
  ShowStr(  'act', actStrs[data.act] );
  ShowStr(  'arg', data.arg );
  ShowStr(  'brg', data.brg );
END Show;

PROCEDURE InitModule();
BEGIN
  Lib2.Fill( data, 0 );
  data.prg := prg;
  data.ver := ver;
  data.ext := ext;
  data.bas := 10;
  InitDir();
  ParseArgs();
  Show();
END InitModule;

BEGIN
  InitModule();
END Config.
